@@ -21,9 +21,11 @@ package org.apache.maven.plugin.antrun;
 
 import java.io.File;
 import java.io.IOException;
+import java.io.LineNumberReader;
 import java.io.StringWriter;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Hashtable;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Properties;
@@ -45,11 +47,13 @@ import org.apache.tools.ant.types.Path;
 import org.codehaus.plexus.configuration.PlexusConfiguration;
 import org.codehaus.plexus.configuration.PlexusConfigurationException;
 import org.codehaus.plexus.util.FileUtils;
+import org.codehaus.plexus.util.IOUtil;
+import org.codehaus.plexus.util.ReaderFactory;
 import org.codehaus.plexus.util.StringUtils;
 
 /**
  * Maven AntRun Mojo.
- *
+ * <br/>
  * This plugin provides the capability of calling Ant tasks
  * from a POM by running the nested ant tasks inside the &lt;tasks/&gt;
  * parameter. It is encouraged to move the actual tasks to
@@ -58,7 +62,7 @@ import org.codehaus.plexus.util.StringUtils;
  *
  * @author <a href="mailto:kenney@apache.org">Kenney Westerhof</a>
  * @author <a href="mailto:vincent.siveton@gmail.com">Vincent Siveton</a>
- * @version $Id: AntRunMojo.java 1005244 2010-10-06 20:53:06Z pgier $
+ * @version $Id: AntRunMojo.java 1190514 2011-10-28 19:27:43Z bimargulies $
  * @goal run
  * @threadSafe
  * @requiresDependencyResolution test
@@ -103,7 +107,7 @@ public class AntRunMojo
 
     /**
      * The Maven project object
-     * 
+     *
      * @parameter expression="${project}"
      * @readonly
      */
@@ -111,14 +115,14 @@ public class AntRunMojo
 
     /**
      * The Maven project helper object
-     * 
+     *
      * @component
      */
     private MavenProjectHelper projectHelper;
 
     /**
      * The plugin dependencies.
-     * 
+     *
      * @parameter expression="${plugin.artifacts}"
      * @required
      * @readonly
@@ -127,7 +131,7 @@ public class AntRunMojo
 
     /**
      * The local Maven repository
-     * 
+     *
      * @parameter expression="${localRepository}"
      * @readonly
      */
@@ -135,7 +139,7 @@ public class AntRunMojo
 
     /**
      * String to prepend to project and dependency property names.
-     * 
+     *
      * @parameter default-value=""
      * @since 1.4
      */
@@ -145,7 +149,7 @@ public class AntRunMojo
      * The xml tag prefix to use for the built in Ant tasks. This prefix needs to be prepended to each task referenced
      * in the antrun target config. For example, a prefix of "mvn" means that the attachartifact task is referenced by
      * "&lt;mvn:attachartifact&gt;" The default value of an empty string means that no prefix is used for the tasks.
-     * 
+     *
      * @parameter default-value=""
      * @since 1.5
      */
@@ -154,7 +158,7 @@ public class AntRunMojo
     /**
      * The name of a property containing the list of all dependency versions. This is used for the removing the versions
      * from the filenames.
-     * 
+     *
      * @parameter default-value="maven.project.dependencies.versions"
      */
     private String versionsPropertyName;
@@ -162,7 +166,7 @@ public class AntRunMojo
     /**
      * The XML for the Ant task. You can add anything you can add between &lt;target&gt; and &lt;/target&gt; in a
      * build.xml.
-     * 
+     *
      * @deprecated Use target instead
      * @parameter
      */
@@ -171,7 +175,7 @@ public class AntRunMojo
     /**
      * The XML for the Ant target. You can add anything you can add between &lt;target&gt; and &lt;/target&gt; in a
      * build.xml.
-     * 
+     *
      * @parameter
      * @since 1.5
      */
@@ -180,7 +184,7 @@ public class AntRunMojo
     /**
      * This folder is added to the list of those folders containing source to be compiled. Use this if your ant script
      * generates source code.
-     * 
+     *
      * @parameter expression="${sourceRoot}"
      * @deprecated Use the build-helper-maven-plugin to bind source directories
      */
@@ -189,18 +193,51 @@ public class AntRunMojo
     /**
      * This folder is added to the list of those folders containing source to be compiled for testing. Use this if your
      * ant script generates test source code.
-     * 
+     *
      * @parameter expression="${testSourceRoot}"
      * @deprecated Use the build-helper-maven-plugin to bind test source directories
      */
     private File testSourceRoot;
 
+    /**
+     * Specifies whether the Antrun execution should be skipped.
+     *
+     * @parameter expression="${maven.antrun.skip}" default-value="false"
+     * @since 1.7
+     */
+    private boolean skip;
+
+    /**
+     * Specifies whether the Ant properties should be propagated to the Maven properties.
+     *
+     * @parameter default-value="false"
+     * @since 1.7
+     */
+    private boolean exportAntProperties;
+    
+    /**
+     * Specifies whether a failure in the ant build leads to a failure of the Maven build.
+     * 
+     * If this value is 'true', the Maven build will proceed even if the ant build fails.
+     * If it is 'false', then the Maven build fails if the ant build fails.
+     * 
+     * @parameter default-value="true"
+     * @since 1.7
+     */
+    private boolean failOnError;
+
     /**
      * @see org.apache.maven.plugin.Mojo#execute()
      */
     public void execute()
         throws MojoExecutionException
     {
+        if ( skip )
+        {
+            getLog().info( "Skipping Antrun execution" );
+            return;
+        }
+
         MavenProject mavenProject = getMavenProject();
 
         if ( tasks != null )
@@ -208,7 +245,7 @@ public class AntRunMojo
             getLog().warn( "Parameter tasks is deprecated, use target instead" );
             target = tasks;
         }
-        
+
         if ( target == null )
         {
             getLog().info( "No ant target defined - SKIPPED" );
@@ -230,7 +267,27 @@ public class AntRunMojo
             DefaultLogger antLogger = new DefaultLogger();
             antLogger.setOutputPrintStream( System.out );
             antLogger.setErrorPrintStream( System.err );
-            antLogger.setMessageOutputLevel( getLog().isDebugEnabled() ? Project.MSG_DEBUG : Project.MSG_INFO );
+            
+            if ( getLog().isDebugEnabled() )
+            {
+                antLogger.setMessageOutputLevel( Project.MSG_DEBUG );
+            }
+            else if ( getLog().isInfoEnabled() )
+            {
+                antLogger.setMessageOutputLevel( Project.MSG_INFO );
+            }
+            else if ( getLog().isWarnEnabled() )
+            {
+                antLogger.setMessageOutputLevel( Project.MSG_WARN );
+            }
+            else if ( getLog().isErrorEnabled() )
+            {
+                antLogger.setMessageOutputLevel( Project.MSG_ERR );
+            }
+            else
+            {
+                antLogger.setMessageOutputLevel( Project.MSG_VERBOSE );
+            }
 
             antProject.addBuildListener( antLogger );
             antProject.setBaseDir( mavenProject.getBasedir() );
@@ -273,6 +330,8 @@ public class AntRunMojo
             {
                 getLog().info( "Executed tasks" );
             }
+
+            copyProperties( antProject, mavenProject );
         }
         catch ( DependencyResolutionRequiredException e )
         {
@@ -280,9 +339,23 @@ public class AntRunMojo
         }
         catch ( BuildException e )
         {
-            throw new MojoExecutionException( "An Ant BuildException has occured: " + e.getMessage(), e );
+            StringBuffer sb = new StringBuffer();
+            sb.append( "An Ant BuildException has occured: " + e.getMessage() );
+            String fragment = findFragment( e );
+            if ( fragment != null )
+            {
+                sb.append( "\n" ).append( fragment );
+            }
+            if ( !failOnError ) 
+            {
+                getLog().info( sb.toString(), e );
+                return; // do not register roots.
+            } else 
+            {
+                throw new MojoExecutionException( sb.toString(), e );
+            }
         }
-        catch ( Exception e )
+        catch ( Throwable e )
         {
             throw new MojoExecutionException( "Error executing ant tasks: " + e.getMessage(), e );
         }
@@ -334,7 +407,7 @@ public class AntRunMojo
 
     /**
      * Copy properties from the maven project to the ant project.
-     * 
+     *
      * @param mavenProject
      * @param antProject
      */
@@ -356,7 +429,10 @@ public class AntRunMojo
         antProject.setProperty( ( propertyPrefix + "project.groupId" ), mavenProject.getGroupId() );
         antProject.setProperty( ( propertyPrefix + "project.artifactId" ), mavenProject.getArtifactId() );
         antProject.setProperty( ( propertyPrefix + "project.name" ), mavenProject.getName() );
-        antProject.setProperty( ( propertyPrefix + "project.description" ), mavenProject.getDescription() );
+        if ( mavenProject.getDescription() != null)
+        {
+            antProject.setProperty( ( propertyPrefix + "project.description" ), mavenProject.getDescription() );            
+        }
         antProject.setProperty( ( propertyPrefix + "project.version" ), mavenProject.getVersion() );
         antProject.setProperty( ( propertyPrefix + "project.packaging" ), mavenProject.getPackaging() );
         antProject.setProperty( ( propertyPrefix + "project.build.directory" ), mavenProject.getBuild().getDirectory() );
@@ -404,16 +480,49 @@ public class AntRunMojo
         }
     }
 
+    /**
+     * Copy properties from the ant project to the maven project.
+     *
+     * @param antProject not null
+     * @param mavenProject not null
+     * @since 1.7
+     */
+    public void copyProperties( Project antProject, MavenProject mavenProject )
+    {
+        if ( !exportAntProperties )
+        {
+            return;
+        }
+
+        getLog().debug( "Propagated Ant properties to Maven properties" );
+        Hashtable antProps = antProject.getProperties();
+
+        Iterator iter = antProps.keySet().iterator();
+        while ( iter.hasNext() )
+        {
+            String key = (String) iter.next();
+
+            Properties mavenProperties = mavenProject.getProperties();
+            if ( mavenProperties.getProperty( key ) != null )
+            {
+                getLog().debug( "Ant property '" + key +"=" +mavenProperties.getProperty( key )
+                                   + "' clashs with an existing Maven property, SKIPPING this Ant property propagation." );
+                continue;
+            }
+            mavenProperties.setProperty( key, antProps.get( key ).toString() );
+        }
+    }
+
     /**
      * Prefix for legacy property format.
      * @deprecated This should only be used for generating the old property format.
      */
     public static final String DEPENDENCY_PREFIX = "maven.dependency.";
-    
+
     /**
      * Returns a property name for a dependency artifact.  The name is in the format
      * maven.dependency.groupId.artifactId[.classifier].type.path
-     * 
+     *
      * @param artifact
      * @return property name
      * @deprecated The dependency conflict ID should be used as the property name.
@@ -428,7 +537,7 @@ public class AntRunMojo
 
     /**
      * Get the current Maven project
-     * 
+     *
      * @return current Maven project
      */
     public MavenProject getMavenProject()
@@ -451,7 +560,7 @@ public class AntRunMojo
 
     /**
      * Write the ant target and surrounding tags to a temporary file
-     * 
+     *
      * @throws PlexusConfigurationException
      */
     private File writeTargetToProjectFile()
@@ -461,7 +570,7 @@ public class AntRunMojo
         StringWriter writer = new StringWriter();
         AntrunXmlPlexusConfigurationWriter xmlWriter = new AntrunXmlPlexusConfigurationWriter();
         xmlWriter.write( target, writer );
-        
+
         StringBuffer antProjectConfig = writer.getBuffer();
 
         // replace deprecated tasks tag with standard Ant target
@@ -479,9 +588,9 @@ public class AntRunMojo
         String xmlns = "";
         if ( ! customTaskPrefix.trim().equals( "" ) )
         {
-            xmlns = "xmlns:" + customTaskPrefix + "=\"" + TASK_URI + "\""; 
+            xmlns = "xmlns:" + customTaskPrefix + "=\"" + TASK_URI + "\"";
         }
-        
+
         final String xmlHeader = "<?xml version=\"1.0\" encoding=\"" + UTF_8 + "\" ?>\n";
         antProjectConfig.insert( 0, xmlHeader );
         final String projectOpen = "<project name=\"maven-antrun-\" default=\"" + antTargetName + "\" " + xmlns +" >\n";
@@ -501,9 +610,9 @@ public class AntRunMojo
     }
 
     /**
-     * Replace text in a StringBuffer.  If the match text is not found, the StringBuffer 
+     * Replace text in a StringBuffer.  If the match text is not found, the StringBuffer
      * is returned unchanged.
-     * 
+     *
      * @param text The string buffer containing the text
      * @param match The string to match and remove
      * @param with The string to insert
@@ -528,4 +637,49 @@ public class AntRunMojo
         return targetName;
     }
 
+    /**
+     * @param buildException not null
+     * @return the fragment XML part where the buildException occurs.
+     * @since 1.7
+     */
+    private String findFragment( BuildException buildException )
+    {
+        if ( buildException == null || buildException.getLocation() == null
+            || buildException.getLocation().getFileName() == null )
+        {
+            return null;
+        }
+
+        File antFile = new File( buildException.getLocation().getFileName() );
+        if ( !antFile.exists() )
+        {
+            return null;
+        }
+
+        LineNumberReader reader = null;
+        try
+        {
+            reader = new LineNumberReader( ReaderFactory.newXmlReader( antFile ) );
+            String line = "";
+            while ( ( line = reader.readLine() ) != null )
+            {
+                if ( reader.getLineNumber() == buildException.getLocation().getLineNumber() )
+                {
+                    return "around Ant part ..." + line.trim() + "... @ " + buildException.getLocation().getLineNumber() + ":"
+                        + buildException.getLocation().getColumnNumber() + " in " + antFile.getAbsolutePath();
+                }
+            }
+        }
+        catch ( Exception e )
+        {
+            getLog().debug( e.getMessage(), e );
+            return null;
+        }
+        finally
+        {
+            IOUtil.close( reader );
+        }
+
+        return null;
+    }
 }
